@@ -46,12 +46,17 @@
 #undef BOOST_GCC_VERSION
 #undef BOOST_GCC_CXX11
 #undef BOOST_GCC
+#undef BOOST_FALLTHROUGH
 
 // Broken in all versions up to 17 (newer versions not tested)
 #if (__INTEL_COMPILER <= 1700) && !defined(BOOST_NO_CXX14_CONSTEXPR)
 #  define BOOST_NO_CXX14_CONSTEXPR
 #endif
 
+#if (__INTEL_COMPILER >= 1800) && (__cplusplus >= 201703)
+#  define BOOST_FALLTHROUGH [[fallthrough]]
+#endif
+
 #endif // defined(_MSC_VER)
 
 #undef BOOST_COMPILER
@@ -558,7 +563,7 @@ template<> struct assert_intrinsic_wchar_t<unsigned short> {};
 //      We don't emit this warning any more, since we have so few
 //      defect macros set anyway (just the one).
 //
-//#     pragma message("Unknown compiler version - please run the configure tests and report the results")
+//#     pragma message("boost: Unknown compiler version - please run the configure tests and report the results")
 #  endif
 #endif
 
